@@ -17,10 +17,12 @@
 #include "ComicBookInfo.h"
 #include "ContinuousPageView.h"
 #include "SimplePageView.h"
-#include "ArchiversConfiguration.h"
+#include "FrameView.h"
+#include "Archivers/ArchiversConfiguration.h"
 #include "ImgArchiveSink.h"
 #include "ImgSinkFactory.h"
 #include "AboutDialog.h"
+#include "ui_DonationDialog.h"
 #include "ComicBookSettings.h"
 #include "History.h"
 #include "ComicBookCfgDialog.h"
@@ -33,6 +35,10 @@
 #include "SystemInfoDialog.h"
 #include "GoToPageWidget.h"
 #include "PageLoaderThread.h"
+#include "RecentFilesMenu.h"
+#include "PrinterThread.h"
+#include <FrameDetectThread.h>
+#include "PrintProgressDialog.h"
 #include <QMenu>
 #include <QStringList>
 #include <QAction>
@@ -45,6 +51,9 @@
 #include <QWidgetAction>
 #include <QList>
 #include <QUrl>
+#include <QPrinter>
+#include <QPrintDialog>
+#include <QApplication>
 #include <QDebug>
 
 using namespace QComicBook;
@@ -57,9 +66,11 @@ ComicMainWindow::ComicMainWindow(QWidget *parent): QMainWindow(parent), view(NUL
     setAttribute(Qt::WA_DeleteOnClose);
     
     cfg = &ComicBookSettings::instance();
-    cfg->restoreGeometry(this);
+    
+    printer = new QPrinter();
 
     pageLoader = new PageLoaderThread();
+    frameDetect = new FrameDetectThread();
 
     //
     // Thumbnails window
@@ -79,6 +90,12 @@ ComicMainWindow::ComicMainWindow(QWidget *parent): QMainWindow(parent), view(NUL
     scaleActions->addAction(actionOriginalSize);
     scaleActions->addAction(actionBestFit);
     connect(scaleActions, SIGNAL(triggered(QAction *)), this, SLOT(setPageSize(QAction *)));
+
+    QActionGroup *viewTypeActions = new QActionGroup(this);
+    viewTypeActions->addAction(actionContinuousView);
+    viewTypeActions->addAction(actionSimpleView);
+    viewTypeActions->addAction(actionFrameView);
+    connect(viewTypeActions, SIGNAL(triggered(QAction *)), this, SLOT(changeViewType(QAction *)));
        
     actionExitFullScreen = new QAction(QString::null, this);
     actionExitFullScreen->setShortcut(tr("Escape"));
@@ -94,6 +111,7 @@ ComicMainWindow::ComicMainWindow(QWidget *parent): QMainWindow(parent), view(NUL
     connect(actionOpenDirectory, SIGNAL(triggered(bool)), this, SLOT(browseDirectory()));
     connect(actionOpenNext, SIGNAL(triggered(bool)), this, SLOT(openNext()));
     connect(actionOpenPrevious, SIGNAL(triggered(bool)), this, SLOT(openPrevious()));
+    connect(actionPrint, SIGNAL(triggered(bool)), this, SLOT(openPrintDialog()));
     connect(actionSavePageAs, SIGNAL(triggered(bool)), this, SLOT(savePageAs()));
     connect(actionShowInfo, SIGNAL(triggered(bool)), this, SLOT(showInfo()));
     connect(actionExitFullScreen, SIGNAL(triggered(bool)), this, SLOT(exitFullscreen()));
@@ -107,6 +125,8 @@ ComicMainWindow::ComicMainWindow(QWidget *parent): QMainWindow(parent), view(NUL
     connect(actionPreviousPage, SIGNAL(triggered(bool)), this, SLOT(prevPage()));   
     connect(actionMangaMode, SIGNAL(toggled(bool)), this, SLOT(toggleJapaneseMode(bool)));        
     connect(actionTwoPages, SIGNAL(toggled(bool)), this, SLOT(toggleTwoPages(bool)));
+    connect(actionNextFrame, SIGNAL(triggered(bool)), this, SLOT(nextFrame()));
+    connect(actionPreviousFrame, SIGNAL(triggered(bool)), this, SLOT(prevFrame()));
 
     //
     // Statusbar
@@ -120,10 +140,10 @@ ComicMainWindow::ComicMainWindow(QWidget *parent): QMainWindow(parent), view(NUL
 
     //
     // File menu
-    menuRecentFiles = new QMenu(tr("Recently opened"), this);
-    menuFile->insertMenu(actionSavePageAs, menuRecentFiles);
-    menuFile->insertSeparator(actionSavePageAs);
-    connect(menuRecentFiles, SIGNAL(triggered(QAction *)), this, SLOT(recentSelected(QAction *)));
+    menuRecentFiles = new RecentFilesMenu(tr("Recently opened"), this, 10);
+    menuFile->insertMenu(actionOpenNext, menuRecentFiles);
+    connect(menuRecentFiles, SIGNAL(selected(const QString &)), this, SLOT(recentSelected(const QString &)));
+    connect(menuRecentFiles, SIGNAL(cleanupRequest()), menuRecentFiles, SLOT(removeAll()));
     connect(actionClose, SIGNAL(triggered()), this, SLOT(closeSink()));
 
     // View menu
@@ -133,11 +153,10 @@ ComicMainWindow::ComicMainWindow(QWidget *parent): QMainWindow(parent), view(NUL
     //
     // Navigation menu
     connect(actionJumpToPage, SIGNAL(triggered()), this, SLOT(showJumpToPage()));
-    connect(actionToggleContinuousScroll, SIGNAL(toggled(bool)), this, SLOT(toggleContinousScroll(bool)));
+    connect(actionContinuousView, SIGNAL(toggled(bool)), this, SLOT(toggleContinousView(bool)));
     actionTwoPages->setChecked(cfg->twoPagesMode());
     actionMangaMode->setChecked(cfg->japaneseMode());
-    actionMangaMode->setDisabled(!cfg->twoPagesMode());
-    actionToggleContinuousScroll->setChecked(cfg->continuousScrolling());
+    actionContinuousView->setChecked(cfg->continuousScrolling());
 
     //
     // Bookmarks menu
@@ -159,6 +178,8 @@ ComicMainWindow::ComicMainWindow(QWidget *parent): QMainWindow(parent), view(NUL
     // Help menu
     connect(actionShowSystemInfo, SIGNAL(triggered()), this, SLOT(showSysInfo()));
     connect(actionShowAbout, SIGNAL(triggered()), this, SLOT(showAbout()));
+    connect(actionAboutQt, SIGNAL(triggered()),  this, SLOT(showAboutQt()));
+    connect(actionAboutDonating, SIGNAL(triggered()),  this, SLOT(showAboutDonating()));
 
     QAction *which = actionOriginalSize;
     switch (cfg->pageSize())
@@ -170,6 +191,15 @@ ComicMainWindow::ComicMainWindow(QWidget *parent): QMainWindow(parent), view(NUL
         case Original:  which = actionOriginalSize; break;
     }
     which->setChecked(true);
+
+    which = actionContinuousView;
+    switch (cfg->viewType())
+    {
+    	case Simple: which = actionSimpleView; break;
+	case Continuous: which = actionContinuousView; break;
+	case Frame: which = actionFrameView; break;
+    }
+    which->setChecked(true);
     
     //
     // copy all menu actions; this is needed for fullscreen mode if menubar is hidden
@@ -186,16 +216,21 @@ ComicMainWindow::ComicMainWindow(QWidget *parent): QMainWindow(parent), view(NUL
     addAction(actionJumpUp);
     
     lastdir = cfg->lastDir();
-    recentfiles = new History(10);
-    *recentfiles = cfg->recentlyOpened();
-    setRecentFilesMenu(*recentfiles);
+    menuRecentFiles->set(cfg->recentlyOpened());
     
     cfg->restoreDockLayout(this);
-    
+    cfg->restoreGeometry(this);
+
+    if (isFullScreen())
+    {
+    	applyFullscreenSettings();
+    }
+
     //connect(cfg, SIGNAL(displaySettingsChanged(const QString &)), this, SLOT(reconfigureDisplay())); ??
     enableComicBookActions(false);
     
     pageLoader->start();
+    frameDetect->start();
 
     thumbnailLoader = new ThumbnailLoaderThread();
     connect(thumbnailLoader, SIGNAL(thumbnailLoaded(const Thumbnail &)), thumbswin, SLOT(setThumbnail(const Thumbnail &)));
@@ -211,14 +246,15 @@ ComicMainWindow::~ComicMainWindow()
 
     saveSettings();        
     
-    delete recentfiles;
     delete bookmarks;
     
+    frameDetect->stop();
     pageLoader->stop();
     thumbnailLoader->stop();
     pageLoader->wait();
     thumbnailLoader->wait();
     
+    delete printer;
     delete pageLoader;
     delete thumbnailLoader;
     
@@ -267,14 +303,35 @@ void ComicMainWindow::setupComicImageView()
         pageLoader->disconnect();
         view->deleteLater();
     }
+
     const ViewProperties props;
-    if (cfg->continuousScrolling())
+    actionNextFrame->setDisabled(true);
+    actionPreviousFrame->setDisabled(true);
+    actionTwoPages->setDisabled(false);
+    actionMangaMode->setDisabled(!cfg->twoPagesMode());
+    switch (cfg->viewType())
     {
-        view = new ContinuousPageView(this, n, props);
-    }
-    else
-    {
-        view = new SimplePageView(this, n, props);
+	case Continuous:
+	        view = new ContinuousPageView(this, n, props);
+		frameDetect->clear();
+		break;
+	case Simple:
+		view = new SimplePageView(this, n, props);
+		frameDetect->clear();
+		break;
+	case Frame:
+		view = new FrameView(this, n, props);
+		actionNextFrame->setDisabled(false);
+		actionPreviousFrame->setDisabled(false);
+		actionTwoPages->setDisabled(true);
+		actionMangaMode->setDisabled(false);
+		//
+		// connect frame processing thread
+		connect(pageLoader, SIGNAL(pageLoaded(const Page &)), frameDetect, SLOT(process(const Page &)));
+		connect(frameDetect, SIGNAL(framesReady(const ComicFrameList &)), view, SLOT(setFrames(const ComicFrameList &)));
+		break;
+	default:
+		break;
     }
     
     setCentralWidget(view);
@@ -322,6 +379,21 @@ void ComicMainWindow::setupComicImageView()
     }
 }
 
+void ComicMainWindow::applyFullscreenSettings()
+{
+    savedToolbarState = toolBar->toggleViewAction()->isChecked() || toolBar->isVisible();
+
+    if (cfg->fullScreenHideMenu())
+	menuBar()->hide();
+    if (cfg->fullScreenHideStatusbar())
+	statusbar->hide();
+    if (cfg->fullScreenHideToolbar())
+    {
+	toolBar->toggleViewAction()->setChecked(false);
+	toolBar->hide();
+    }
+}
+
 void ComicMainWindow::enableComicBookActions(bool f)
 {
         //
@@ -332,6 +404,7 @@ void ComicMainWindow::enableComicBookActions(bool f)
         actionOpenNext->setEnabled(x);
         actionOpenPrevious->setEnabled(x);
 	actionSavePageAs->setEnabled(x);
+        actionPrint->setEnabled(x);
 
         //
         // view menu
@@ -350,6 +423,8 @@ void ComicMainWindow::enableComicBookActions(bool f)
         actionForwardPage->setEnabled(f);
         actionPageTop->setEnabled(f);
         actionPageBottom->setEnabled(f);
+	actionNextFrame->setEnabled(f && actionFrameView->isChecked());
+	actionPreviousFrame->setEnabled(f && actionFrameView->isChecked());
 
         //
         // bookmarks menu
@@ -412,7 +487,7 @@ void ComicMainWindow::toggleScrollbars(bool f)
     view->enableScrollbars(f);
 }
 
-void ComicMainWindow::toggleContinousScroll(bool f)
+void ComicMainWindow::toggleContinousView(bool f)
 {
     cfg->continuousScrolling(f);
     setupComicImageView();
@@ -477,36 +552,14 @@ void ComicMainWindow::updateCaption()
     setWindowTitle(c);
 }
 
-void ComicMainWindow::setRecentFilesMenu(const History &hist)
+void ComicMainWindow::recentSelected(const QString &fname) 
 {
-        QStringList list = hist.getAll();
-        //workaround for crash when removing recent items via clear()
-        //when it's called from recentSelected handler.
-        QList<QAction *> actions = menuRecentFiles->actions();
-        foreach (QAction *a, actions) {
-            a->deleteLater();
-        }
-
-        foreach (const QString &s, list)
-        {
-            QAction *a = menuRecentFiles->addAction(Utility::shortenPath(s, "...", 64));
-            a->setData(s);
-        }
-}
-
-void ComicMainWindow::recentSelected(QAction *action) 
-{
-    const QString fname(action->data().toString());
-    if (fname != QString::null)
+    QFileInfo finfo(fname);
+    if (!finfo.exists())
     {
-        QFileInfo finfo(fname);
-        if (!finfo.exists())
-        {
-            recentfiles->remove(fname);
-            menuRecentFiles->removeAction(action);
-        }
-        open(fname, 0);
+        menuRecentFiles->remove(fname);
     }
+    open(fname, 0);
 }
 
 void ComicMainWindow::pageLoaded(const Page &page)
@@ -532,8 +585,7 @@ void ComicMainWindow::sinkReady(const QString &path)
 	statusbar->setShown(actionToggleStatusbar->isChecked() && !(isFullScreen() && cfg->fullScreenHideStatusbar())); //applies back user's statusbar&toolbar preferences
 	//toolbar->setShown(actiontoggleToolbar->isOn() && !(isFullScreen() && cfg->fullScreenHideToolbar()));
 
-        recentfiles->append(path);
-        setRecentFilesMenu(*recentfiles);
+        menuRecentFiles->add(path);
 
         enableComicBookActions(true);
         updateCaption();
@@ -652,18 +704,8 @@ void ComicMainWindow::toggleFullScreen()
         }
         else
         {
-		savedToolbarState = toolBar->toggleViewAction()->isChecked();
-
-                if (cfg->fullScreenHideMenu())
-                        menuBar()->hide();
-                if (cfg->fullScreenHideStatusbar())
-                        statusbar->hide();
-                if (cfg->fullScreenHideToolbar())
-		{
-			toolBar->toggleViewAction()->setChecked(false);
-                        toolBar->hide();
-		}
-                showFullScreen();
+	    applyFullscreenSettings();
+	    showFullScreen();
         }
 }
 
@@ -674,11 +716,12 @@ void ComicMainWindow::exitFullscreen()
                 menuBar()->show();
                 if (actionToggleStatusbar->isChecked())
                         statusbar->show();
-		toolBar->toggleViewAction()->setChecked(savedToolbarState);
-		if (savedToolbarState)
+		if (!toolBar->isVisible())
+		{
+		    toolBar->toggleViewAction()->setChecked(savedToolbarState);
+		    if (savedToolbarState)
 			toolBar->show();
-		else
-			toolBar->hide();
+		}
 		showNormal();
         }
 }
@@ -701,6 +744,16 @@ void ComicMainWindow::prevPage()
     }
 }
 
+void ComicMainWindow::prevFrame()
+{
+    view->prevFrame();
+}
+
+void ComicMainWindow::nextFrame()
+{
+    view->nextFrame();
+}
+
 void ComicMainWindow::prevPageBottom()
 {
     if (currpage > 0)
@@ -817,6 +870,29 @@ void ComicMainWindow::showAbout()
         win->show();
 }
 
+void ComicMainWindow::showAboutDonating(bool startup)
+{
+    QDialog dlg;
+    Ui::DonationDialog d;
+    d.setupUi(&dlg);
+    if (startup)
+    {
+    	d.closeBtn->hide();
+    }
+    else
+    {
+	d.remindBtn->hide();
+	d.donateBtn->hide();
+    }
+    if (dlg.exec() == QDialog::Accepted)
+    {
+	if (startup)
+	{
+		cfg->showDonationDialog(false);
+	}
+    }
+}
+
 void ComicMainWindow::showConfigDialog()
 {
         ComicBookCfgDialog *d = new ComicBookCfgDialog(this, cfg);
@@ -833,6 +909,11 @@ void ComicMainWindow::showJumpToPage(const QString &number)
                 win.exec();
         }
 }
+			
+void ComicMainWindow::showAboutQt()
+{
+	QApplication::aboutQt();	
+}
 
 void ComicMainWindow::closeSink()
 {
@@ -840,6 +921,7 @@ void ComicMainWindow::closeSink()
 
     if (sink)
     {
+	frameDetect->clear();
         pageLoader->cancelAll();
         pageLoader->setSink(NULL);
         thumbnailLoader->cancelAll();
@@ -902,6 +984,23 @@ void ComicMainWindow::savePageAs()
 	}
 }
 
+void ComicMainWindow::openPrintDialog()
+{
+    QPrintDialog printdlg(printer, this);
+    printdlg.setMinMax(1, sink->numOfImages());
+    if (printdlg.exec() == QDialog::Accepted)
+    {
+        PrintProgressDialog *progressdlg = new PrintProgressDialog(this);
+        printThread = new PrinterThread(sink, printer, printdlg.printRange(), printdlg.fromPage(), printdlg.toPage());
+        connect(printThread, SIGNAL(printing(int)), progressdlg, SLOT(setPage(int)));
+        connect(printThread, SIGNAL(finished()), progressdlg, SLOT(close()));
+        connect(printThread, SIGNAL(finished()), this, SLOT(printingFinished()));
+        connect(progressdlg, SIGNAL(abort()), printThread, SLOT(abort()));
+        progressdlg->show();
+        printThread->start();
+    }
+}
+
 void ComicMainWindow::bookmarkSelected(QAction *action) 
 {
         Bookmark b;
@@ -929,13 +1028,30 @@ void ComicMainWindow::bookmarkSelected(QAction *action)
                 }
         }
 }
+			
+void ComicMainWindow::changeViewType(QAction *action)
+{
+    if (action == actionContinuousView)
+    {
+	cfg->viewType(Continuous);
+    }
+    else if (action == actionSimpleView)
+    {
+	cfg->viewType(Simple);
+    }
+    else // actionFrameView
+    {
+	cfg->viewType(Frame);
+    }
+    setupComicImageView();
+}
 
 void ComicMainWindow::saveSettings()
 {
         cfg->saveGeometry(this);
         cfg->saveDockLayout(this);
         cfg->lastDir(lastdir);
-        cfg->recentlyOpened(*recentfiles);
+        cfg->recentlyOpened(menuRecentFiles->get());
         
         cfg->showStatusbar(actionToggleStatusbar->isChecked());
 
@@ -948,3 +1064,8 @@ void ComicMainWindow::reconfigureDisplay()
     view->showPageNumbers(cfg->embedPageNumbers());
     view->setBackground(cfg->background());
 }
+
+void ComicMainWindow::printingFinished()
+{
+    printThread->deleteLater();
+}
