@@ -1,7 +1,7 @@
 /*
  * This file is a part of QComicBook.
  *
- * Copyright (C) 2005-2006 Pawel Stolowski <pawel.stolowski@wp.pl>
+ * Copyright (C) 2005-2010 Pawel Stolowski <stolowski@gmail.com>
  *
  * QComicBook is free software; you can redestribute it and/or modify it
  * under terms of GNU General Public License by Free Software Foundation.
@@ -17,12 +17,12 @@
 
 #include <QMainWindow>
 #include "ui_ComicMainWindow.h"
-#include "History.h"
 
 class QMenu;
 class QLabel;
 class QKeyEvent;
 class QAction;
+class QPrinter;
 
 namespace QComicBook
 {
@@ -32,10 +32,12 @@ namespace QComicBook
 	class ThumbnailsWindow;
 	class Bookmarks;
 	class StatusBar;
-        class PageLoaderThread;
-        class Page;
-        class ThumbnailLoaderThread;
-	using Utility::History;
+	class PageLoaderThread;
+	class Page;
+	class ThumbnailLoaderThread;
+	class RecentFilesMenu;
+	class PrinterThread;
+	class FrameDetectThread;
 
 	//! The main window of QComicBook.
 	class ComicMainWindow: public QMainWindow, private Ui::ComicMainWindow
@@ -46,21 +48,22 @@ namespace QComicBook
 			ImgDirSink *sink;
 			PageViewBase *view;
 			ThumbnailsWindow *thumbswin;
-			History *recentfiles;
 			Bookmarks *bookmarks;
 			StatusBar *statusbar;
 			ComicBookSettings *cfg;
 			int currpage; //!<current page number
 					
 			bool savedToolbarState;
-			QMenu *context_menu;
-			QMenu *menuRecentFiles;
-                        QAction *actionToggleThumbnails;
-                        QAction *actionExitFullScreen;
+			RecentFilesMenu *menuRecentFiles;
+			QAction *actionToggleThumbnails;
+			QAction *actionExitFullScreen;
 			QLabel *pageinfo; //!<page info displayed in right-click context menu
 			QString lastdir; //!<last opened directory for Open File/Directory dialog
-                        PageLoaderThread *pageLoader;
-                        ThumbnailLoaderThread *thumbnailLoader;
+			PageLoaderThread *pageLoader;
+			ThumbnailLoaderThread *thumbnailLoader;
+			QPrinter *printer;
+			PrinterThread *printThread;
+			FrameDetectThread *frameDetect;
                         		
 		protected:
 			virtual void dragEnterEvent(QDragEnterEvent *e);
@@ -74,26 +77,31 @@ namespace QComicBook
 
                         void setupContextMenu();
 			void setupComicImageView();
+			void applyFullscreenSettings();
 
 		protected slots:
-                        void pageLoaded(const Page &page);
-                        void pageLoaded(const Page &page1, const Page &page2);
+			void pageLoaded(const Page &page);
+			void pageLoaded(const Page &page1, const Page &page2);
 			void sinkReady(const QString &path);
 			void sinkError(int code);
 			void updateCaption();
-			void setRecentFilesMenu(const History &hist);
-			void recentSelected(QAction *action);
+			void recentSelected(const QString &fname);
 			void bookmarkSelected(QAction *action);
 			void thumbnailsWindowShown();
 			void savePageAs();
-                        void reconfigureDisplay();
-                        void currentPageChanged(int n);
-                        void setPageSize(QAction *action);
+			void reconfigureDisplay();
+			void currentPageChanged(int n);
+			void setPageSize(QAction *action);
+			void changeViewType(QAction *action);
+			void printingFinished();
+			void showAboutQt();
 
 		public slots:
 			void firstPage();
 			void lastPage();
 			void nextPage();
+			void nextFrame();
+			void prevFrame();
 			void prevPage();
 			void prevPageBottom();
 			void forwardPages();
@@ -117,9 +125,11 @@ namespace QComicBook
 			void toggleScrollbars(bool f);
 			void toggleTwoPages(bool f);
 			void toggleFullScreen();
-			void toggleContinousScroll(bool f);
+			void toggleContinousView(bool f);
 			void toggleJapaneseMode(bool f);
 			void reloadPage();
+			void openPrintDialog();
+			void showAboutDonating(bool startup=false);
 
 		public:
 			ComicMainWindow(QWidget *parent);
